@@ -776,6 +776,7 @@ void linphone_core_get_local_ip(LinphoneCore *lc, const char *dest, char *result
 		strncpy(result,linphone_core_get_nat_address(lc),LINPHONE_IPADDR_SIZE);
 		return;
 	}
+	if (dest==NULL) dest="87.98.157.38"; /*a public IP address*/
 	if (linphone_core_get_local_ip_for(dest,result)==0)
 		return;
 	/*else fallback to exosip routine that will attempt to find the most realistic interface */
@@ -1274,7 +1275,7 @@ LinphoneProxyConfig * linphone_core_lookup_known_proxy(LinphoneCore *lc, const c
 	return found_cfg;
 }
 
-static void fix_contact(osip_message_t *msg, const char *localip, LinphoneProxyConfig *dest_proxy){
+static void fix_contact(LinphoneCore *lc, osip_message_t *msg, const char *localip, LinphoneProxyConfig *dest_proxy){
 	osip_contact_t *ctt=NULL;
 	const char *ip=NULL;
 	int port=5060;
@@ -1287,7 +1288,7 @@ static void fix_contact(osip_message_t *msg, const char *localip, LinphoneProxyC
 			linphone_proxy_config_get_contact(dest_proxy,&ip,&port);
 		}else{
 			ip=localip;
-			port=linphone_core_get_sip_port(dest_proxy->lc);
+			port=linphone_core_get_sip_port(lc);
 		}
 		if (ip!=NULL){
 			osip_free(ctt->url->host);
@@ -1365,7 +1366,7 @@ int linphone_core_invite(LinphoneCore *lc, const char *url)
 	/*try to be best-effort in giving real local or routable contact address,
 	except when the user choosed to override the ipaddress */
 	if (linphone_core_get_firewall_policy(lc)!=LINPHONE_POLICY_USE_NAT_ADDRESS)
-		fix_contact(invite,lc->call->localip,dest_proxy);
+		fix_contact(lc,invite,lc->call->localip,dest_proxy);
 
 	barmsg=ortp_strdup_printf("%s %s", _("Contacting"), real_url);
 	lc->vtable.display_status(lc,barmsg);
@@ -1508,16 +1509,12 @@ void linphone_core_init_media_streams(LinphoneCore *lc){
 	lc->audiostream=audio_stream_new(linphone_core_get_audio_port(lc),linphone_core_ipv6_enabled(lc));
 	if (linphone_core_echo_limiter_enabled(lc)){
 		const char *type=lp_config_get_string(lc->config,"sound","el_type","mic");
-		float gain=lp_config_get_float(lc->config,"sound","mic_gain",-1);
 		if (strcasecmp(type,"mic")==0)
 			audio_stream_enable_echo_limiter(lc->audiostream,ELControlMic);
 		else if (strcasecmp(type,"speaker")==0)
 			audio_stream_enable_echo_limiter(lc->audiostream,ELControlSpeaker);
-		if (gain!=-1){
-			audio_stream_enable_gain_control(lc->audiostream,TRUE);
-		}
-		
 	}
+	audio_stream_enable_gain_control(lc->audiostream,TRUE);
 	if (linphone_core_echo_cancelation_enabled(lc)){
 		int len,delay,framesize;
 		len=lp_config_get_int(lc->config,"sound","ec_tail_len",0);
@@ -1564,11 +1561,14 @@ static void parametrize_equalizer(LinphoneCore *lc, AudioStream *st){
 
 static void post_configure_audio_streams(LinphoneCore *lc){
 	AudioStream *st=lc->audiostream;
-	if (st->volrecv && st->volsend){
+	float gain=lp_config_get_float(lc->config,"sound","mic_gain",-1);
+	if (gain!=-1)
+		audio_stream_set_mic_gain(st,gain);
+	if (linphone_core_echo_limiter_enabled(lc)){
 		float speed=lp_config_get_float(lc->config,"sound","el_speed",-1);
 		float thres=lp_config_get_float(lc->config,"sound","el_thres",-1);
 		float force=lp_config_get_float(lc->config,"sound","el_force",-1);
-		float gain=lp_config_get_float(lc->config,"sound","mic_gain",-1);
+		int sustain=lp_config_get_int(lc->config,"sound","el_sustain",-1);
 		MSFilter *f=NULL;
 		if (st->el_type==ELControlMic){
 			f=st->volsend;
@@ -1579,9 +1579,6 @@ static void post_configure_audio_streams(LinphoneCore *lc){
 			f=st->volrecv;
 			if (speed==-1) speed=0.02;
 			if (force==-1) force=5;
-		}else {
-			ms_fatal("Should not happen");
-			return;
 		}
 		if (speed!=-1)
 			ms_filter_call_method(f,MS_VOLUME_SET_EA_SPEED,&speed);
@@ -1589,8 +1586,9 @@ static void post_configure_audio_streams(LinphoneCore *lc){
 			ms_filter_call_method(f,MS_VOLUME_SET_EA_THRESHOLD,&thres);
 		if (force!=-1)
 			ms_filter_call_method(f,MS_VOLUME_SET_EA_FORCE,&force);
-		if (gain!=-1)
-			audio_stream_set_mic_gain(st,gain);
+		if (sustain!=-1)
+			ms_filter_call_method(f,MS_VOLUME_SET_EA_SUSTAIN,&sustain);
+		
 	}
 	parametrize_equalizer(lc,st);
 	if (lc->vtable.dtmf_received!=NULL){
@@ -1747,7 +1745,7 @@ int linphone_core_accept_call(LinphoneCore *lc, const char *url)
 	/*try to be best-effort in giving real local or routable contact address,
 	except when the user choosed to override the ipaddress */
 	if (linphone_core_get_firewall_policy(lc)!=LINPHONE_POLICY_USE_NAT_ADDRESS)
-		fix_contact(msg,call->localip,NULL);
+		fix_contact(lc,msg,call->localip,NULL);
 	/*if a sdp answer is computed, send it, else send an offer */
 	sdpmesg=call->sdpctx->answerstr;
 	if (sdpmesg==NULL){
@@ -2073,6 +2071,13 @@ bool_t linphone_core_echo_limiter_enabled(const LinphoneCore *lc){
 	return lc->sound_conf.ea;
 }
 
+void linphone_core_mute_mic(LinphoneCore *lc, bool_t val){
+	if (lc->audiostream!=NULL){
+		 audio_stream_set_mic_gain(lc->audiostream,
+			(val==TRUE) ? 0 : 1.0);
+	}
+}
+
 void linphone_core_enable_agc(LinphoneCore *lc, bool_t val){
 	lc->sound_conf.agc=val;
 }
