@@ -319,6 +319,7 @@ int linphone_proxy_config_done(LinphoneProxyConfig *obj);
 void linphone_proxy_config_enable_publish(LinphoneProxyConfig *obj, bool_t val);
 bool_t linphone_proxy_config_is_registered(const LinphoneProxyConfig *obj);
 const char *linphone_proxy_config_get_domain(const LinphoneProxyConfig *cfg);
+
 #define linphone_proxy_config_get_route(obj)  ((obj)->reg_route)
 #define linphone_proxy_config_get_identity(obj)	((obj)->reg_identity)
 #define linphone_proxy_config_publish_enabled(obj) ((obj)->publish)
@@ -332,6 +333,7 @@ LinphoneProxyConfig *linphone_proxy_config_new_from_config_file(struct _LpConfig
 void linphone_proxy_config_write_to_config_file(struct _LpConfig* config,LinphoneProxyConfig *obj, int index);
 void linphone_proxy_config_set_sip_setup(LinphoneProxyConfig *cfg, const char *type);
 SipSetupContext *linphone_proxy_config_get_sip_setup_context(LinphoneProxyConfig *cfg);
+SipSetup *linphone_proxy_config_get_sip_setup(LinphoneProxyConfig *cfg);
 
 typedef struct _LinphoneAccountCreator{
 	struct _LinphoneCore *lc;
@@ -724,6 +726,8 @@ bool_t linphone_core_echo_limiter_enabled(const LinphoneCore *lc);
 void linphone_core_enable_agc(LinphoneCore *lc, bool_t val);
 bool_t linphone_core_agc_enabled(const LinphoneCore *lc);
 
+void linphone_core_mute_mic(LinphoneCore *lc, bool_t muted);
+
 void linphone_core_set_presence_info(LinphoneCore *lc,int minutes_away,const char *contact,LinphoneOnlineStatus os);
 
 LinphoneOnlineStatus linphone_core_get_presence_info(const LinphoneCore *lc);
