@@ -69,14 +69,18 @@ void linphone_gtk_check_for_new_version(void);
 const char *linphone_gtk_get_lang(const char *config_file);
 void linphone_gtk_set_lang(const char *code);
 SipSetupContext* linphone_gtk_get_default_sip_setup_context(void);
-void linphone_gtk_show_buddy_lookup_window(SipSetupContext *ctx);
+GtkWidget * linphone_gtk_show_buddy_lookup_window(SipSetupContext *ctx);
+void linphone_gtk_buddy_lookup_set_keyword(GtkWidget *w, const char *kw);
 void * linphone_gtk_wait(LinphoneCore *lc, void *ctx, LinphoneWaitingState ws, const char *purpose, float progress);
 gchar *linphone_gtk_get_display_name(const char *sip_uri);
 
-/*functions controlling the in-call view*/
+void linphone_gtk_show_directory_search(void);
+
+/*functions controlling the different views*/
 void linphone_gtk_show_in_call_view(void);
 void linphone_gtk_show_idle_view(void);
 void linphone_gtk_in_call_view_set_calling(const char *uri);
-void linphone_gtk_in_call_view_set_in_call(const char *uri);
+void linphone_gtk_in_call_view_set_in_call(void);
 void linphone_gtk_in_call_view_update_duration(int duration);
 void linphone_gtk_in_call_view_terminate(const char *error_msg);
+void linphone_gtk_show_login_frame(LinphoneProxyConfig *cfg);
