@@ -76,6 +76,11 @@ static LinphoneCoreVTable vtable={
 static gboolean verbose=0;
 static gboolean auto_answer = 0;
 static gchar * addr_to_call = NULL;
+static gboolean iconified=FALSE;
+#ifdef WIN32
+static gchar *workingdir=NULL;
+#endif
+
 static GOptionEntry linphone_options[]={
 	{
 		.long_name="verbose",
@@ -84,20 +89,36 @@ static GOptionEntry linphone_options[]={
 		.arg_data= (gpointer)&verbose,
 		.description=N_("log to stdout some debug information while running.")
 	},
-	{				/* zsd addition */
+	{
+		.long_name="iconified",
+		.short_name= '\0',
+		.arg=G_OPTION_ARG_NONE,
+		.arg_data= (gpointer)&iconified,
+		.description=N_("Start only in the system tray, do not show the main interface.")
+	},
+	{
 	    .long_name = "call",
 	    .short_name = 'c',
 	    .arg = G_OPTION_ARG_STRING,
 	    .arg_data = &addr_to_call,
 	    .description = N_("address to call right now")
 	},
-	{				/* zsd addition */
+	{
 	    .long_name = "auto-answer",
 	    .short_name = 'a',
 	    .arg = G_OPTION_ARG_NONE,
 	    .arg_data = (gpointer) & auto_answer,
 	    .description = N_("if set automatically answer incoming calls")
 	},
+#ifdef WIN32
+	{				/* zsd addition */
+	    .long_name = "workdir",
+	    .short_name = '\0',
+	    .arg = G_OPTION_ARG_STRING,
+	    .arg_data = (gpointer) & workingdir,
+	    .description = N_("Specifiy a working directory (should be the base of the installation, eg: c:\\Program Files\\Linphone)")
+	},
+#endif
 	{0}
 };
 
@@ -366,6 +387,7 @@ static void update_video_title(){
 }
 
 static gboolean linphone_gtk_iterate(LinphoneCore *lc){
+	static gboolean first_time=TRUE;
 	unsigned long id;
 	static unsigned long previd=0;
 	static gboolean in_iterate=FALSE;
@@ -374,6 +396,12 @@ static gboolean linphone_gtk_iterate(LinphoneCore *lc){
 	if (in_iterate) return TRUE;
 	in_iterate=TRUE;
 	linphone_core_iterate(lc);
+	if (first_time){
+		/*after the first call to iterate, SipSetupContexts should be ready, so take actions:*/
+		linphone_gtk_show_directory_search();
+		first_time=FALSE;
+	}
+
 	id=linphone_core_get_native_video_window_id(lc);
 	if (id!=previd || video_needs_update){
 		GdkWindow *w;
@@ -394,10 +422,15 @@ static gboolean linphone_gtk_iterate(LinphoneCore *lc){
 		}
 	}
 	if (addr_to_call!=NULL){
-		GtkWidget *uri_bar=linphone_gtk_get_widget(linphone_gtk_get_main_window(),"uribar");
-		gtk_entry_set_text(GTK_ENTRY(uri_bar),addr_to_call);
-		addr_to_call=NULL;
-		linphone_gtk_start_call(uri_bar);	
+		/*make sure we are not showing the login screen*/
+		GtkWidget *mw=linphone_gtk_get_main_window();
+		GtkWidget *login_frame=linphone_gtk_get_widget(mw,"login_frame");
+		if (!GTK_WIDGET_VISIBLE(login_frame)){
+			GtkWidget *uri_bar=linphone_gtk_get_widget(mw,"uribar");
+			gtk_entry_set_text(GTK_ENTRY(uri_bar),addr_to_call);
+			addr_to_call=NULL;
+			linphone_gtk_start_call(uri_bar);
+		}
 	}
 	in_iterate=FALSE;
 	return TRUE;
@@ -551,7 +584,7 @@ void linphone_gtk_accept_call(GtkWidget *button){
 	g_object_set_data(G_OBJECT(linphone_gtk_get_main_window()),"incoming_call",NULL);
 	gtk_widget_destroy(gtk_widget_get_toplevel(button));
 	linphone_gtk_call_started(linphone_gtk_get_main_window());
-	linphone_gtk_in_call_view_set_in_call(linphone_core_get_remote_uri(lc));
+	linphone_gtk_in_call_view_set_in_call();
 	linphone_gtk_show_in_call_view();
 }
 
@@ -578,8 +611,10 @@ void linphone_gtk_enable_self_view(GtkWidget *w){
 void linphone_gtk_used_identity_changed(GtkWidget *w){
 	int active=gtk_combo_box_get_active(GTK_COMBO_BOX(w));
 	char *sel=gtk_combo_box_get_active_text(GTK_COMBO_BOX(w));
-	if (sel && strlen(sel)>0) //avoid a dummy "changed" at gui startup
+	if (sel && strlen(sel)>0){ //avoid a dummy "changed" at gui startup
 		linphone_core_set_default_proxy_index(linphone_gtk_get_core(),(active==0) ? -1 : (active-1));
+		linphone_gtk_show_directory_search();
+	}
 }
 
 static void linphone_gtk_show_main_window(){
@@ -758,7 +793,7 @@ static void linphone_gtk_general_state(LinphoneCore *lc, LinphoneGeneralState *g
 	switch(gstate->new_state){
 		case GSTATE_CALL_OUT_CONNECTED:
 		case GSTATE_CALL_IN_CONNECTED:
-			linphone_gtk_in_call_view_set_in_call(linphone_core_get_remote_uri(lc));
+			linphone_gtk_in_call_view_set_in_call();
 		break;
 		case GSTATE_CALL_ERROR:
 			linphone_gtk_call_terminated(gstate->message);
@@ -777,8 +812,8 @@ static void icon_popup_menu(GtkStatusIcon *status_icon, guint button, guint acti
 }
 
 void linphone_gtk_open_browser(const char *url){
-	gtk_show_uri(NULL,url,GDK_CURRENT_TIME,NULL);
-#if 0
+	/*in gtk 2.16, gtk_show_uri does not work...*/
+	/*gtk_show_uri(NULL,url,GDK_CURRENT_TIME,NULL);*/
 #ifdef WIN32
 	ShellExecute(0,"open",url,NULL,NULL,1);
 #else
@@ -786,7 +821,6 @@ void linphone_gtk_open_browser(const char *url){
 	snprintf(cl,sizeof(cl),"/usr/bin/x-www-browser %s",url);
 	g_spawn_command_line_async(cl,NULL);
 #endif
-#endif
 }
 
 void linphone_gtk_link_to_website(GtkWidget *item){
@@ -954,8 +988,21 @@ static void linphone_gtk_configure_main_window(){
 		gtk_widget_hide(linphone_gtk_get_widget(w,"run_assistant"));
 }
 
+void linphone_gtk_manage_login(void){
+	LinphoneCore *lc=linphone_gtk_get_core();
+	LinphoneProxyConfig *cfg=NULL;
+	linphone_core_get_default_proxy(lc,&cfg);
+	if (cfg){
+		SipSetup *ss=linphone_proxy_config_get_sip_setup(cfg);
+		if (ss && (sip_setup_get_capabilities(ss) & SIP_SETUP_CAP_LOGIN)){
+			linphone_gtk_show_login_frame(cfg);
+		}
+	}
+}
+
 static void linphone_gtk_init_main_window(){
 	linphone_gtk_configure_main_window();
+	linphone_gtk_manage_login();
 	load_uri_history();
 	linphone_gtk_load_identities();
 	linphone_gtk_set_my_presence(linphone_core_get_presence_info(linphone_gtk_get_core()));
@@ -1028,7 +1075,6 @@ static void linphone_gtk_refer_received(LinphoneCore *lc, const char *refer_to){
 	linphone_gtk_start_call(linphone_gtk_get_main_window());
 }
 
-
 int main(int argc, char *argv[]){
 #ifdef ENABLE_NLS
 	void *p;
@@ -1042,6 +1088,9 @@ int main(int argc, char *argv[]){
 	config_file=linphone_gtk_get_config_file();
 
 #ifdef WIN32
+	if (workingdir!=NULL)
+		_chdir(workingdir);
+
 	/*workaround for windows: sometimes LANG is defined to an integer value, not understood by gtk */
 	if ((lang=getenv("LANG"))!=NULL){
 		if (atoi(lang)!=0){
@@ -1103,7 +1152,8 @@ int main(int argc, char *argv[]){
 	gtk_timeout_add(30,(GtkFunction)linphone_gtk_check_logs,(gpointer)NULL);
 	linphone_gtk_init_main_window();
 	linphone_gtk_init_status_icon();
-	linphone_gtk_show_main_window();
+	if (!iconified)
+		linphone_gtk_show_main_window();
 	linphone_gtk_check_for_new_version();
 
 	gtk_main();
