@@ -82,7 +82,11 @@ typedef enum MSFilterId{
 	MS_MIRE_ID,
 	MS_VFW_ID,
 	MS_ICE_ID,
-	MS_VOID_SINK_ID
+	MS_VOID_SINK_ID,
+	MS_DSCAP_ID,
+	MS_AQ_READ_ID,
+	MS_AQ_WRITE_ID,
+	MS_EQUALIZER_ID
 } MSFilterId;
 
 
