@@ -30,6 +30,8 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "mediastreamer2/msfileplayer.h"
 #include "mediastreamer2/msfilerec.h"
 #include "mediastreamer2/msvolume.h"
+#include "mediastreamer2/msequalizer.h"
+#include "mediastreamer2/msspeexec.h"
 
 #ifdef INET6
 	#include <sys/types.h>
@@ -57,6 +59,7 @@ void audio_stream_free(AudioStream *stream)
 	if (stream->ec!=NULL)	ms_filter_destroy(stream->ec);
 	if (stream->volrecv!=NULL) ms_filter_destroy(stream->volrecv);
 	if (stream->volsend!=NULL) ms_filter_destroy(stream->volsend);
+	if (stream->equalizer!=NULL) ms_filter_destroy(stream->equalizer);
 	if (stream->ticker!=NULL) ms_ticker_destroy(stream->ticker);
 	ms_free(stream);
 }
@@ -220,6 +223,7 @@ int audio_stream_start_full(AudioStream *stream, RtpProfile *profile, const char
 	if (captcard!=NULL) stream->soundread=ms_snd_card_create_reader(captcard);
 	else {
 		stream->soundread=ms_filter_new(MS_FILE_PLAYER_ID);
+		stream->resampler=ms_filter_new(MS_RESAMPLE_ID);
 		if (infile!=NULL) audio_stream_play(stream,infile);
 	}
 	if (playcard!=NULL) stream->soundwrite=ms_snd_card_create_writer(playcard);
@@ -245,9 +249,15 @@ int audio_stream_start_full(AudioStream *stream, RtpProfile *profile, const char
 	if (use_ec) {
 		stream->ec=ms_filter_new(MS_SPEEX_EC_ID);
 		ms_filter_call_method(stream->ec,MS_FILTER_SET_SAMPLE_RATE,&pt->clock_rate);
+		if (stream->ec_tail_len!=0)
+			ms_filter_call_method(stream->ec,MS_SPEEX_EC_SET_TAIL_LENGTH,&stream->ec_tail_len);
+		if (stream->ec_delay!=0)
+			ms_filter_call_method(stream->ec,MS_SPEEX_EC_SET_DELAY,&stream->ec_delay);
+		if (stream->ec_framesize!=0)
+			ms_filter_call_method(stream->ec,MS_SPEEX_EC_SET_FRAME_SIZE,&stream->ec_framesize);
 	}
 
-	if (stream->el_type!=ELInactive || stream->use_gc){
+	if (stream->el_type!=ELInactive || stream->use_gc || stream->use_ng){
 		stream->volsend=ms_filter_new(MS_VOLUME_ID);
 		stream->volrecv=ms_filter_new(MS_VOLUME_ID);
 		if (stream->el_type!=ELInactive){
@@ -255,6 +265,17 @@ int audio_stream_start_full(AudioStream *stream, RtpProfile *profile, const char
 				ms_filter_call_method(stream->volrecv,MS_VOLUME_SET_PEER,stream->volsend);
 			else ms_filter_call_method(stream->volsend,MS_VOLUME_SET_PEER,stream->volrecv);
 		}
+		if (stream->use_ng){
+			int tmp=1;
+			ms_filter_call_method(stream->volsend,MS_VOLUME_ENABLE_NOISE_GATE,&tmp);
+		}
+	}
+
+	if (stream->use_agc){
+		int tmp=1;
+		if (stream->volsend==NULL)
+			stream->volsend=ms_filter_new(MS_VOLUME_ID);
+		ms_filter_call_method(stream->volsend,MS_VOLUME_ENABLE_AGC,&tmp);
 	}
 
 	/* give the sound filters some properties */
@@ -275,12 +296,19 @@ int audio_stream_start_full(AudioStream *stream, RtpProfile *profile, const char
 	if (pt->send_fmtp!=NULL) ms_filter_call_method(stream->encoder,MS_FILTER_ADD_FMTP, (void*)pt->send_fmtp);
 	if (pt->recv_fmtp!=NULL) ms_filter_call_method(stream->decoder,MS_FILTER_ADD_FMTP,(void*)pt->recv_fmtp);
 	
+	/*create the equalizer*/
+	stream->equalizer=ms_filter_new(MS_EQUALIZER_ID);
+	tmp=stream->eq_active;
+	ms_filter_call_method(stream->equalizer,MS_EQUALIZER_SET_ACTIVE,&tmp);
+
 	/* and then connect all */
 	/* tip: draw yourself the picture if you don't understand */
 	
 	/*sending graph*/
 	ms_connection_helper_start(&h);
 	ms_connection_helper_link(&h,stream->soundread,-1,0);
+	if (stream->resampler)
+		ms_connection_helper_link(&h,stream->resampler,0,0);
 	if (stream->ec)
 		ms_connection_helper_link(&h,stream->ec,1,1);
 	if (stream->volsend)
@@ -293,6 +321,8 @@ int audio_stream_start_full(AudioStream *stream, RtpProfile *profile, const char
 	ms_connection_helper_link(&h,stream->rtprecv,-1,0);
 	ms_connection_helper_link(&h,stream->decoder,0,0);
 	ms_connection_helper_link(&h,stream->dtmfgen,0,0);
+	if (stream->equalizer)
+		ms_connection_helper_link(&h,stream->equalizer,0,0);
 	if (stream->volrecv)
 		ms_connection_helper_link(&h,stream->volrecv,0,0);
 	if (stream->ec)
@@ -355,8 +385,15 @@ void audio_stream_set_rtcp_information(AudioStream *st, const char *cname, const
 
 void audio_stream_play(AudioStream *st, const char *name){
 	if (ms_filter_get_id(st->soundread)==MS_FILE_PLAYER_ID){
+		int from_rate=0, to_rate=0;
 		ms_filter_call_method_noarg(st->soundread,MS_FILE_PLAYER_CLOSE);
 		ms_filter_call_method(st->soundread,MS_FILE_PLAYER_OPEN,(void*)name);
+		ms_filter_call_method(st->soundread,MS_FILTER_GET_SAMPLE_RATE,&from_rate);
+		ms_filter_call_method(st->rtpsend,MS_FILTER_GET_SAMPLE_RATE,&to_rate);
+		if (st->resampler){
+			ms_filter_call_method(st->resampler,MS_FILTER_SET_SAMPLE_RATE,&from_rate);
+			ms_filter_call_method(st->resampler,MS_FILTER_SET_OUTPUT_SAMPLE_RATE,&to_rate);
+		}
 		ms_filter_call_method_noarg(st->soundread,MS_FILE_PLAYER_START);
 	}else{
 		ms_error("Cannot play file: the stream hasn't been started with"
@@ -382,6 +419,8 @@ AudioStream *audio_stream_new(int locport, bool_t ipv6){
 	stream->rtpsend=ms_filter_new(MS_RTP_SEND_ID);
 	stream->play_dtmfs=TRUE;
 	stream->use_gc=FALSE;
+	stream->use_agc=FALSE;
+	stream->use_ng=FALSE;
 	return stream;
 }
 
@@ -398,6 +437,12 @@ void audio_stream_set_relay_session_id(AudioStream *stream, const char *id){
 	ms_filter_call_method(stream->rtpsend, MS_RTP_SEND_SET_RELAY_SESSION_ID,(void*)id);
 }
 
+void audio_stream_set_echo_canceler_params(AudioStream *st, int tail_len_ms, int delay_ms, int framesize){
+	st->ec_tail_len=tail_len_ms;
+	st->ec_delay=delay_ms;
+	st->ec_framesize=framesize;
+}
+
 void audio_stream_enable_echo_limiter(AudioStream *stream, EchoLimiterType type){
 	stream->el_type=type;
 }
@@ -406,6 +451,14 @@ void audio_stream_enable_gain_control(AudioStream *stream, bool_t val){
 	stream->use_gc=val;
 }
 
+void audio_stream_enable_automatic_gain_control(AudioStream *stream, bool_t val){
+	stream->use_agc=val;
+}
+
+void audio_stream_enable_noise_gate(AudioStream *stream, bool_t val){
+	stream->use_ng=val;
+}
+
 void audio_stream_set_mic_gain(AudioStream *stream, float gain){
 	if (stream->volsend){
 		ms_filter_call_method(stream->volsend,MS_VOLUME_SET_GAIN,&gain);
@@ -413,6 +466,24 @@ void audio_stream_set_mic_gain(AudioStream *stream, float gain){
 			"Use audio_stream_enable_gain_control() before starting the stream.");
 }
 
+void audio_stream_enable_equalizer(AudioStream *stream, bool_t enabled){
+	stream->eq_active=enabled;
+	if (stream->equalizer){
+		int tmp=enabled;
+		ms_filter_call_method(stream->equalizer,MS_EQUALIZER_SET_ACTIVE,&tmp);
+	}
+}
+
+void audio_stream_equalizer_set_gain(AudioStream *stream, int frequency, float gain, int freq_width){
+	if (stream->equalizer){
+		MSEqualizerGain d;
+		d.frequency=frequency;
+		d.gain=gain;
+		d.width=freq_width;
+		ms_filter_call_method(stream->equalizer,MS_EQUALIZER_SET_GAIN,&d);
+	}
+}
+
 void audio_stream_stop(AudioStream * stream)
 {
 	if (stream->ticker){
@@ -425,6 +496,8 @@ void audio_stream_stop(AudioStream * stream)
 		/*dismantle the outgoing graph*/
 		ms_connection_helper_start(&h);
 		ms_connection_helper_unlink(&h,stream->soundread,-1,0);
+		if (stream->resampler!=NULL)
+			ms_connection_helper_unlink(&h,stream->resampler,0,0);
 		if (stream->ec!=NULL)
 			ms_connection_helper_unlink(&h,stream->ec,1,1);
 		if (stream->volsend!=NULL)
@@ -437,6 +510,8 @@ void audio_stream_stop(AudioStream * stream)
 		ms_connection_helper_unlink(&h,stream->rtprecv,-1,0);
 		ms_connection_helper_unlink(&h,stream->decoder,0,0);
 		ms_connection_helper_unlink(&h,stream->dtmfgen,0,0);
+		if (stream->equalizer)
+			ms_connection_helper_unlink(&h,stream->equalizer,0,0);
 		if (stream->volrecv!=NULL)
 			ms_connection_helper_unlink(&h,stream->volrecv,0,0);
 		if (stream->ec!=NULL)
