@@ -40,6 +40,9 @@ void rfc3984_init(Rfc3984Context *ctx){
 	ms_queue_init(&ctx->q);
 	ctx->m=NULL;
 	ctx->maxsz=ms_get_payload_max_size();
+	ctx->mode=0;
+	ctx->last_ts=0x943FEA43;/*some random value*/
+	ctx->stap_a_allowed=TRUE;
 }
 
 void rfc3984_set_max_payload_size(Rfc3984Context *ctx, int size){
@@ -112,39 +115,63 @@ static void frag_nalu_and_send(MSQueue *rtpq, uint32_t ts, mblk_t *nalu, bool_t
 	send_packet(rtpq,ts,m,marker);
 }
 
+static void rfc3984_pack_mode_0(Rfc3984Context *ctx, MSQueue *naluq, MSQueue *rtpq, uint32_t ts){
+	mblk_t *m;
+	bool_t end;
+	int size;
+	while((m=ms_queue_get(naluq))!=NULL){
+		end=ms_queue_empty(naluq);
+		size=m->b_wptr-m->b_rptr;
+		if (size>ctx->maxsz){
+			ms_warning("This H264 packet does not fit into mtu: size=%i",size);
+		}
+		send_packet(rtpq,ts,m,end);
+	}
+}
+
 /*process NALUs and pack them into rtp payloads */
-void rfc3984_pack(Rfc3984Context *ctx, MSQueue *naluq, MSQueue *rtpq, uint32_t ts){
+static void rfc3984_pack_mode_1(Rfc3984Context *ctx, MSQueue *naluq, MSQueue *rtpq, uint32_t ts){
 	mblk_t *m,*prevm=NULL;
 	int prevsz=0,sz;
 	bool_t end;
 	while((m=ms_queue_get(naluq))!=NULL){
 		end=ms_queue_empty(naluq);
 		sz=m->b_wptr-m->b_rptr;
-		if (prevm!=NULL){
-			if ((prevsz+sz)<(ctx->maxsz-2)){
-				prevm=concat_nalus(prevm,m);
+		if (ctx->stap_a_allowed){
+			if (prevm!=NULL){
+				if ((prevsz+sz)<(ctx->maxsz-2)){
+					prevm=concat_nalus(prevm,m);
+					m=NULL;
+					prevsz+=sz+2;/*+2 for the stapa size field*/
+					continue;
+				}else{
+					/*send prevm packet: either single nal or STAP-A*/
+					if (prevm->b_cont!=NULL){
+						ms_debug("Sending STAP-A");
+					}else
+						ms_debug("Sending previous msg as single NAL");
+					send_packet(rtpq,ts,prevm,FALSE);
+					prevm=NULL;
+					prevsz=0;
+				}
+			}
+			if (sz<(ctx->maxsz/2)){
+				/*try to aggregate it with next packet*/
+				prevm=m;
+				prevsz=sz+3; /*STAP-A header + size*/
 				m=NULL;
-				prevsz+=sz+2;/*+2 for the stapa size field*/
-				continue;
 			}else{
-				/*send prevm packet: either single nal or STAP-A*/
-				if (prevm->b_cont!=NULL){
-					ms_debug("Sending STAP-A");
-				}else
-					ms_debug("Sending previous msg as single NAL");
-				send_packet(rtpq,ts,prevm,FALSE);
-				prevm=NULL;
-				prevsz=0;
+				
+				/*send as single nal or FU-A*/
+				if (sz>ctx->maxsz){
+					ms_debug("Sending FU-A packets");
+					frag_nalu_and_send(rtpq,ts,m,end, ctx->maxsz);
+				}else{
+					ms_debug("Sending Single NAL");
+					send_packet(rtpq,ts,m,end);
+				}
 			}
-		}
-		if (sz<(ctx->maxsz/2)){
-			/*try to aggregate it with next packet*/
-			prevm=m;
-			prevsz=sz+3; /*STAP-A header + size*/
-			m=NULL;
 		}else{
-			
-			/*send as single nal or FU-A*/
 			if (sz>ctx->maxsz){
 				ms_debug("Sending FU-A packets");
 				frag_nalu_and_send(rtpq,ts,m,end, ctx->maxsz);
@@ -160,7 +187,6 @@ void rfc3984_pack(Rfc3984Context *ctx, MSQueue *naluq, MSQueue *rtpq, uint32_t t
 	}
 }
 
-
 static mblk_t * aggregate_fua(Rfc3984Context *ctx, mblk_t *im){
 	mblk_t *om=NULL;
 	uint8_t fu_header;
@@ -202,11 +228,24 @@ static mblk_t * aggregate_fua(Rfc3984Context *ctx, mblk_t *im){
 void rfc3984_unpack(Rfc3984Context *ctx, mblk_t *im, MSQueue *out){
 	uint8_t type=nal_header_get_type(im->b_rptr);
 	uint8_t *p;
-	
+	int marker = mblk_get_marker_info(im);
+	uint32_t ts=mblk_get_timestamp_info(im);
+
+	if (ctx->last_ts!=ts){
+		/*a new frame is arriving, in case the marker bit was not set in previous frame, output it now*/
+		/* unless this is a FU-A (workarond some other apps bugs)*/
+		ctx->last_ts=ts;
+		if (ctx->m==NULL){
+			while(!ms_queue_empty(&ctx->q)){
+				ms_queue_put(out,ms_queue_get(&ctx->q));
+			}
+		}
+	}
+
 	if (im->b_cont) msgpullup(im,-1);
 
 	if (type==TYPE_STAP_A){
-		ms_message("Receiving STAP-A");
+		ms_debug("Receiving STAP-A");
 		/*split into nalus*/
 		uint16_t sz;
 		uint8_t *buf=(uint8_t*)&sz;
@@ -229,15 +268,23 @@ void rfc3984_unpack(Rfc3984Context *ctx, mblk_t *im, MSQueue *out){
 		}
 		freemsg(im);
 	}else if (type==TYPE_FU_A){
-		ms_message("Receiving FU-A");
+		ms_debug("Receiving FU-A");
 		mblk_t *o=aggregate_fua(ctx,im);
 		if (o) ms_queue_put(&ctx->q,o);
 	}else{
+		if (ctx->m){
+			/*discontinued FU-A, purge it*/
+			freemsg(ctx->m);
+			ctx->m=NULL;
+		}
 		/*single nal unit*/
-		ms_message("Receiving single NAL");
+		ms_debug("Receiving single NAL");
 		ms_queue_put(&ctx->q,im);
 	}
-	if (mblk_get_marker_info(im)){
+
+	if (marker){
+		ctx->last_ts=ts;
+		ms_debug("Marker bit set");
 		/*end of frame, output everything*/
 		while(!ms_queue_empty(&ctx->q)){
 			ms_queue_put(out,ms_queue_get(&ctx->q));
@@ -246,8 +293,29 @@ void rfc3984_unpack(Rfc3984Context *ctx, mblk_t *im, MSQueue *out){
 }
 
 
+void rfc3984_pack(Rfc3984Context *ctx, MSQueue *naluq, MSQueue *rtpq, uint32_t ts){
+	switch(ctx->mode){
+		case 0:
+			rfc3984_pack_mode_0(ctx,naluq,rtpq,ts);
+			break;
+		case 1:
+			rfc3984_pack_mode_1(ctx,naluq,rtpq,ts);
+			break;
+		default:
+			ms_error("Bad or unsupported mode %i",ctx->mode);
+	}
+}
+
 void rfc3984_uninit(Rfc3984Context *ctx){
 	ms_queue_flush(&ctx->q);
 	if (ctx->m) freemsg(ctx->m);
 	ctx->m=NULL;
 }
+
+void rfc3984_set_mode(Rfc3984Context *ctx, int mode){
+	ctx->mode=mode;
+}
+
+void rfc3984_enable_stap_a(Rfc3984Context *ctx, bool_t yesno){
+	ctx->stap_a_allowed=yesno;
+}
