@@ -226,9 +226,11 @@ static void sdl_display_uninit(MSDisplay *obj){
 		SDL_FreeSurface(sdl_screen);
 		sdl_screen=NULL;
 	}
+#ifdef __linux
 	/*purge the event queue before leaving*/
 	for(i=0;SDL_PollEvent(&event) && i<100;++i){
 	}
+#endif
 	sdl_show_window(FALSE);
 }
 
@@ -664,6 +666,10 @@ static void video_out_uninit(MSFilter *f){
 static void video_out_prepare(MSFilter *f){
 	VideoOut *obj=(VideoOut*)f->data;
 	if (obj->display==NULL){
+		if (default_display_desc==NULL){
+			ms_error("No default display built in !");
+			return;
+		}
 		obj->display=ms_display_new(default_display_desc);
 		obj->own_display=TRUE;
 	}
