@@ -547,6 +547,16 @@ static int v4w_configure_videodevice(V4wState *s)
 		ms_message("Selected Size: %ix%i.", MS_VIDEO_SIZE_VGA_W, MS_VIDEO_SIZE_VGA_H);
 	else if (try_format_size(s, s->pix_fmt, MS_VIDEO_SIZE_QVGA_W, MS_VIDEO_SIZE_QVGA_H, &pPinCategory)==0)
 		ms_message("Selected Size: %ix%i.", MS_VIDEO_SIZE_QVGA_W, MS_VIDEO_SIZE_QVGA_H);
+	else if (try_format_size(s, s->pix_fmt, MS_VIDEO_SIZE_QQVGA_W, MS_VIDEO_SIZE_QQVGA_H, &pPinCategory)==0)
+		ms_message("Selected Size: %ix%i.", MS_VIDEO_SIZE_QQVGA_W, MS_VIDEO_SIZE_QQVGA_H);
+	else if (try_format_size(s, s->pix_fmt, MS_VIDEO_SIZE_NS1_W, MS_VIDEO_SIZE_NS1_H, &pPinCategory)==0)
+		ms_message("Selected Size: %ix%i.", MS_VIDEO_SIZE_NS1_W, MS_VIDEO_SIZE_NS1_H);
+	else if (try_format_size(s, s->pix_fmt, MS_VIDEO_SIZE_720P_W, MS_VIDEO_SIZE_720P_H, &pPinCategory)==0)
+		ms_message("Selected Size: %ix%i.", MS_VIDEO_SIZE_720P_W, MS_VIDEO_SIZE_720P_H);
+	else if (try_format_size(s, s->pix_fmt, MS_VIDEO_SIZE_800X600_W, MS_VIDEO_SIZE_800X600_H, &pPinCategory)==0)
+		ms_message("Selected Size: %ix%i.", MS_VIDEO_SIZE_800X600_W, MS_VIDEO_SIZE_800X600_H);
+	else if (try_format_size(s, s->pix_fmt, MS_VIDEO_SIZE_1024_W, MS_VIDEO_SIZE_1024_H, &pPinCategory)==0)
+		ms_message("Selected Size: %ix%i.", MS_VIDEO_SIZE_1024_W, MS_VIDEO_SIZE_1024_H);
 	else
 	{
 		ms_error("No supported size found for format.");
@@ -712,6 +722,16 @@ static int v4w_open_videodevice(V4wState *s)
 		ms_message("Selected Size: %ix%i.", MS_VIDEO_SIZE_VGA_W, MS_VIDEO_SIZE_VGA_H);
 	else if (try_format_size(s, s->pix_fmt, MS_VIDEO_SIZE_QVGA_W, MS_VIDEO_SIZE_QVGA_H, &pPinCategory)==0)
 		ms_message("Selected Size: %ix%i.", MS_VIDEO_SIZE_QVGA_W, MS_VIDEO_SIZE_QVGA_H);
+	else if (try_format_size(s, s->pix_fmt, MS_VIDEO_SIZE_QQVGA_W, MS_VIDEO_SIZE_QQVGA_H, &pPinCategory)==0)
+		ms_message("Selected Size: %ix%i.", MS_VIDEO_SIZE_QQVGA_W, MS_VIDEO_SIZE_QQVGA_H);
+	else if (try_format_size(s, s->pix_fmt, MS_VIDEO_SIZE_NS1_W, MS_VIDEO_SIZE_NS1_H, &pPinCategory)==0)
+		ms_message("Selected Size: %ix%i.", MS_VIDEO_SIZE_NS1_W, MS_VIDEO_SIZE_NS1_H);
+	else if (try_format_size(s, s->pix_fmt, MS_VIDEO_SIZE_720P_W, MS_VIDEO_SIZE_720P_H, &pPinCategory)==0)
+		ms_message("Selected Size: %ix%i.", MS_VIDEO_SIZE_720P_W, MS_VIDEO_SIZE_720P_H);
+	else if (try_format_size(s, s->pix_fmt, MS_VIDEO_SIZE_800X600_W, MS_VIDEO_SIZE_800X600_H, &pPinCategory)==0)
+		ms_message("Selected Size: %ix%i.", MS_VIDEO_SIZE_800X600_W, MS_VIDEO_SIZE_800X600_H);
+	else if (try_format_size(s, s->pix_fmt, MS_VIDEO_SIZE_1024_W, MS_VIDEO_SIZE_1024_H, &pPinCategory)==0)
+		ms_message("Selected Size: %ix%i.", MS_VIDEO_SIZE_1024_W, MS_VIDEO_SIZE_1024_H);
 	else
 	{
 		ms_error("No supported size found for format.");
